@@ -77,11 +77,8 @@ class Interface:
         self.Dataset = None
         self.text_file = None
         self.params = None
-        self.Facets = None
         self.preprocessing_folder = None
         self.postprocessing_folder = None
-        self.reconstruction_files = None
-        self.strain_output_file = None
 
         # Init tabs
         self.TabStartup = TabStartup()
@@ -206,7 +203,7 @@ class Interface:
                 flag_interact=self.TabPreprocess.flag_interact,
                 background_plot=self.TabPreprocess.background_plot,
                 unused_label_centering=self.TabPreprocess.unused_label_centering,
-                centering_method=self.TabPreprocess.centering_method,
+                centering_method_reciprocal_space=self.TabPreprocess.centering_method_reciprocal_space,
                 bragg_peak=self.TabPreprocess.bragg_peak,
                 fix_size=self.TabPreprocess.fix_size,
                 center_fft=self.TabPreprocess.center_fft,
@@ -228,7 +225,7 @@ class Interface:
                 save_to_vti=self.TabPreprocess.save_to_vti,
                 save_as_int=self.TabPreprocess.save_as_int,
                 unused_label_preprocess=self.TabPreprocess.unused_label_preprocess,
-                init_para=self.TabPreprocess.init_para,
+                run_preprocess=self.TabPreprocess.run_preprocess,
             )
 
             self.init_data_frame_tab_gui = interactive(
@@ -258,11 +255,14 @@ class Interface:
                     support_update_period=self.TabPhaseRetrieval.support_update_period,
                     support_smooth_width=self.TabPhaseRetrieval.support_smooth_width,
                     support_post_expand=self.TabPhaseRetrieval.support_post_expand,
+                    support_method=self.TabPhaseRetrieval.support_method,
+                    support_autocorrelation_threshold=self.TabPhaseRetrieval.support_autocorrelation_threshold,
                     unused_label_psf=self.TabPhaseRetrieval.unused_label_psf,
                     psf=self.TabPhaseRetrieval.psf,
                     psf_model=self.TabPhaseRetrieval.psf_model,
                     fwhm=self.TabPhaseRetrieval.fwhm,
                     eta=self.TabPhaseRetrieval.eta,
+                    psf_filter=self.TabPhaseRetrieval.psf_filter,
                     update_psf=self.TabPhaseRetrieval.update_psf,
                     unused_label_algo=self.TabPhaseRetrieval.unused_label_algo,
                     nb_hio=self.TabPhaseRetrieval.nb_hio,
@@ -275,14 +275,17 @@ class Interface:
                     nb_run_keep=self.TabPhaseRetrieval.nb_run_keep,
                     unused_label_options=self.TabPhaseRetrieval.unused_label_options,
                     live_plot=self.TabPhaseRetrieval.live_plot,
-                    # zero_mask TODO = self.TabPhaseRetrieval.    # TODO,
-                    # crop_output TODO = self.TabPhaseRetrieval.    # TODO,
+                    plot_axis=self.TabPhaseRetrieval.plot_axis,
+                    verbose=self.TabPhaseRetrieval.verbose,
+                    rebin=self.TabPhaseRetrieval.rebin,
                     positivity=self.TabPhaseRetrieval.positivity,
                     beta=self.TabPhaseRetrieval.beta,
                     detwin=self.TabPhaseRetrieval.detwin,
-                    rebin=self.TabPhaseRetrieval.rebin,
-                    verbose=self.TabPhaseRetrieval.verbose,
+                    calc_llk=self.TabPhaseRetrieval.calc_llk,
                     pixel_size_detector=self.TabPhaseRetrieval.pixel_size_detector,
+                    unused_label_mask_options=self.TabPhaseRetrieval.unused_label_mask_options,
+                    zero_mask=self.TabPhaseRetrieval.zero_mask,
+                    mask_interp=self.TabPhaseRetrieval.mask_interp,
                     unused_label_phase_retrieval=self.TabPhaseRetrieval.unused_label_phase_retrieval,
                     run_phase_retrieval=self.TabPhaseRetrieval.run_phase_retrieval,
                     unused_label_run_pynx_tools=self.TabPhaseRetrieval.unused_label_run_pynx_tools,
@@ -312,7 +315,7 @@ class Interface:
                 phase_offset=self.TabPostprocess.phase_offset,
                 phase_offset_origin=self.TabPostprocess.phase_offset_origin,
                 offset_method=self.TabPostprocess.offset_method,
-                centering_method=self.TabPostprocess.centering_method,
+                centering_method_direct_space=self.TabPostprocess.centering_method_direct_space,
                 unused_label_refraction=self.TabPostprocess.unused_label_refraction,
                 correct_refraction=self.TabPostprocess.correct_refraction,
                 optical_path_method=self.TabPostprocess.optical_path_method,
@@ -353,7 +356,7 @@ class Interface:
                 apodization_alpha=self.TabPostprocess.apodization_alpha,
                 unused_label_strain=self.TabPostprocess.unused_label_strain,
                 strain_folder=self.TabPostprocess.strain_folder,
-                reconstruction_files=self.TabPostprocess.reconstruction_files,
+                reconstruction_file=self.TabPostprocess.reconstruction_file,
                 init_postprocess_parameters=self.TabPostprocess.init_postprocess_parameters,
             )
 
@@ -423,7 +426,7 @@ class Interface:
                 # Set tab names
                 self.window.set_title(0, "Startup")
                 self.window.set_title(1, "Detector")
-                self.window.set_title(2, "Setup")
+                self.window.set_title(2, "Instrument")
                 self.window.set_title(3, "Preprocess")
                 self.window.set_title(4, "Metadata")
                 self.window.set_title(5, "Phase retrieval")
@@ -469,7 +472,7 @@ class Interface:
                 # Set tab names
                 self.window.set_title(0, "Startup")
                 self.window.set_title(1, "Detector")
-                self.window.set_title(2, "Setup")
+                self.window.set_title(2, "Instrument")
                 self.window.set_title(3, "Preprocess")
                 self.window.set_title(4, "Metadata")
                 self.window.set_title(5, "Postprocess")
@@ -483,7 +486,7 @@ class Interface:
                 self.root_folder_handler, names="value")
             self.TabStartup.run_dir_init.observe(
                 self.init_handler, names="value")
-            self.TabPreprocess.init_para.observe(
+            self.TabPreprocess.run_preprocess.observe(
                 self.preprocess_handler, names="value")
 
         # Display the final window
@@ -507,30 +510,30 @@ class Interface:
         if not change.new:
             for w in self.TabStartup.children[:-1]:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = False
+                    for wc in w.children:
+                        wc.disabled = False
                 else:
                     w.disabled = False
 
             for w in self.TabDetector.children + self.TabInstrument.children + self.TabPreprocess.children:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = True
+                    for wc in w.children:
+                        wc.disabled = True
                 else:
                     w.disabled = True
 
         if change.new:
             for w in self.TabStartup.children[:-1]:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = True
+                    for wc in w.children:
+                        wc.disabled = True
                 else:
                     w.disabled = True
 
             for w in self.TabDetector.children + self.TabInstrument.children + self.TabPreprocess.children:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = False
+                    for wc in w.children:
+                        wc.disabled = False
                 else:
                     w.disabled = False
 
@@ -538,7 +541,7 @@ class Interface:
             self.TabPreprocess.beamline_handler(
                 change=self.TabPreprocess.beamline.value)
             self.TabPreprocess.bragg_peak_centering_handler(
-                change=self.TabPreprocess.centering_method.value)
+                change=self.TabPreprocess.centering_method_reciprocal_space.value)
             self.TabPreprocess.reload_data_handler(
                 change=self.TabPreprocess.reload_previous.value)
 
@@ -552,8 +555,8 @@ class Interface:
 
             for w in self.TabPreprocess.children[:-1]:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = False
+                    for wc in w.children:
+                        wc.disabled = False
                 else:
                     w.disabled = False
 
@@ -561,7 +564,7 @@ class Interface:
             self.TabPreprocess.beamline_handler(
                 change=self.TabPreprocess.beamline.value)
             self.TabPreprocess.bragg_peak_centering_handler(
-                change=self.TabPreprocess.centering_method.value)
+                change=self.TabPreprocess.centering_method_reciprocal_space.value)
             self.TabPreprocess.reload_data_handler(
                 change=self.TabPreprocess.reload_previous.value)
 
@@ -570,7 +573,7 @@ class Interface:
 
             for w in self.TabPreprocess.children[:-1]:
                 if isinstance(w, widgets.VBox) or isinstance(w, widgets.HBox):
-                    for w in w.children:
-                        w.disabled = True
+                    for wc in w.children:
+                        wc.disabled = True
                 else:
                     w.disabled = True
