@@ -1,6 +1,7 @@
 import warnings
 import numpy as np
 import h5py as h5
+import os
 
 from tornado.ioloop import PeriodicCallback
 
@@ -53,7 +54,11 @@ class Plotter:
         :param filename: path to data, supported files extensions are .cxi,
          .npy or .npz
         :param data_array: a np.ndarray
-        :param plot: either '2D', 'slices', '3D' or False
+        :param plot: str in [
+            '2D', 'slices', 'contour_slices',
+            'sum_slices', 'sum_contour_slices',
+            '3D', False,
+         ]
         :param log: True to have a logarithmic scale
          False to have a linear scale
         :param cmap: matplotlib cmap used for plot, default 'YlGnBu_r'
@@ -75,39 +80,73 @@ class Plotter:
         self.interact_scale = False
 
         # Get data array from any of the supported files
-        if isinstance(data, str):
+        if os.path.isfile(data):
             self.filename = data
             self.get_data_array()
 
         elif isinstance(data, np.ndarray):
             self.data_array = data
-            # Plot data
-            if self.plot == "2D":
-                self.plot_data(figsize=self.figsize, fontsize=self.fontsize,
-                               log=self.log, cmap=self.cmap, title=self.title)
-
-            elif self.plot == "slices" and self.data_array.ndim == 3:
-                self.plot_3d_slices(figsize=None, log=self.log, cmap=self.cmap,
-                                    fontsize=self.fontsize, title=self.title)
-
-            elif self.plot == "3D" and self.data_array.ndim == 3:
-                ThreeDViewer(self.data_array)
-
-            else:
-                print(
-                    "#########################################################"
-                    "########################################################\n"
-                    f"Loaded data array\n"
-                    f"\tNb of dimensions: {self.data_array.ndim}\n"
-                    f"\tShape: {self.data_array.shape}\n"
-                    "\t'plot' keyword accepted values: ('2D', 'slices', 3D')"
-                    "\n#########################################################"
-                    "########################################################"
-                )
+            self.init_plot()
 
         else:
             print(
-                "Please provide either a filename (arg filename) or directly an np.ndarray (arg data_array).")
+                "Please provide either a valid filename (arg filename)"
+                " or directly an np.ndarray (arg data_array)."
+            )
+
+    def init_plot(self):
+        if self.plot == "2D":
+            plot_data(
+                data_array=self.data_array,
+                figsize=self.figsize, fontsize=self.fontsize,
+                log=self.log, cmap=self.cmap, title=self.title,
+            )
+
+        elif self.plot == "slices" and self.data_array.ndim == 3:
+            plot_3d_slices(
+                data_array=self.data_array,
+                fontsize=self.fontsize, title=self.title,
+                figsize=None, log=self.log, cmap=self.cmap,
+                contour=False, sum_over_axis=False,
+            )
+
+        elif self.plot == "contour_slices" and self.data_array.ndim == 3:
+            plot_3d_slices(
+                data_array=self.data_array,
+                fontsize=self.fontsize, title=self.title,
+                figsize=None, log=self.log, cmap=self.cmap,
+                contour=True, sum_over_axis=False,
+            )
+
+        elif self.plot == "sum_slices" and self.data_array.ndim == 3:
+            plot_3d_slices(
+                data_array=self.data_array,
+                fontsize=self.fontsize, title=self.title,
+                figsize=None, log=self.log, cmap=self.cmap,
+                contour=False, sum_over_axis=True,
+            )
+
+        elif self.plot == "sum_contour_slices" and self.data_array.ndim == 3:
+            plot_3d_slices(
+                data_array=self.data_array,
+                fontsize=self.fontsize, title=self.title,
+                figsize=None, log=self.log, cmap=self.cmap,
+                contour=True, sum_over_axis=True,
+            )
+
+        elif self.plot == "3D" and self.data_array.ndim == 3:
+            ThreeDViewer(self.data_array)
+
+        else:
+            print(
+                "#########################################################"
+                "########################################################\n"
+                f"Loaded data array\n"
+                f"\tNb of dimensions: {self.data_array.ndim}\n"
+                f"\tShape: {self.data_array.shape}\n"
+                "\n#########################################################"
+                "########################################################"
+            )
 
     def get_data_array(self):
         """Get numpy array from file.
@@ -164,28 +203,7 @@ class Plotter:
                             """)
 
             # Plot data
-            if self.plot == "2D":
-                self.plot_data(figsize=self.figsize, fontsize=self.fontsize,
-                               log=self.log, cmap=self.cmap, title=self.title)
-
-            elif self.plot == "slices" and self.data_array.ndim == 3:
-                self.plot_3d_slices(figsize=None, log=self.log, fontsize=self.fontsize,
-                                    cmap=self.cmap, title=self.title)
-
-            elif self.plot == "3D" and self.data_array.ndim == 3:
-                ThreeDViewer(self.data_array)
-
-            else:
-                print(
-                    "#########################################################"
-                    "########################################################\n"
-                    f"Loaded data array from {self.filename}\n"
-                    f"\tNb of dimensions: {self.data_array.ndim}\n"
-                    f"\tShape: {self.data_array.shape}\n"
-                    "\t'plot' keyword accepted values: ('2D', 'slices', 3D')"
-                    "\n#########################################################"
-                    "########################################################"
-                )
+            self.init_plot()
 
         # Need to select data array interactively
         elif self.filename.endswith(".npz"):
@@ -207,32 +225,7 @@ class Plotter:
                         print("Key not valid, is this an array ?")
 
                     # Plot data
-                    if self.plot == "2D":
-                        print(
-                            "\n#############################################"
-                            f"\nArray shape (x, y, z): {self.data_array.shape}\n"
-                            "##############################################"
-                        )
-                        self.plot_data(fontsize=self.fontsize, title=self.title,
-                                       figsize=self.figsize, log=self.log,
-                                       cmap=self.cmap)
-
-                    elif self.plot == "slices" and self.data_array.ndim == 3:
-                        self.plot_3d_slices(fontsize=self.fontsize, title=self.title,
-                                            figsize=None, log=self.log,
-                                            cmap=self.cmap)
-
-                    elif self.plot == "3D" and self.data_array.ndim == 3:
-                        ThreeDViewer(self.data_array)
-
-                    else:
-                        print(
-                            "\n#############################################"
-                            f"\nLoaded data array from {self.filename}\n"
-                            f"\tNb of dimensions: {self.data_array.ndim}\n"
-                            f"\tShape: {self.data_array.shape}\n"
-                            "##############################################\n"
-                        )
+                    self.init_plot()
 
             except ValueError:
                 print("Could not load data.")
@@ -240,39 +233,13 @@ class Plotter:
         else:
             print("Data type not supported.")
 
-    def plot_data(self, **kwargs):
-        """Run plot_data function with class arguments."""
-        for k, v in kwargs.items():
-            setattr(self, k, v)
-
-        plot_data(
-            data_array=self.data_array,
-            figsize=self.figsize,
-            fontsize=self.fontsize,
-            cmap=self.cmap,
-            title=self.title,
-        )
-
-    def plot_3d_slices(self, **kwargs):
-        """Run plot_3d_slices function with class arguments."""
-        for k, v in kwargs.items():
-            setattr(self, k, v)
-
-        plot_3d_slices(
-            data_array=self.data_array,
-            figsize=self.figsize,
-            log=self.log,
-            cmap=self.cmap,
-            title=self.title,
-        )
-
 
 class ThreeDViewer(widgets.Box):
     """
     Widget to display 3D objects from CDI optimisation, loaded from a result
     CXI file or a mode file.
 
-    Quickly adapted from @Vincent Favre Nicolin (ESRF)
+    Simplified from the widgets class in PyNX @Vincent Favre Nicolin (ESRF)
     """
 
     def __init__(self, input_file=None, html_width=None):
@@ -423,7 +390,7 @@ class ThreeDViewer(widgets.Box):
 
         try:
             verts, faces, _, _ = marching_cubes(
-                abs(self.d),
+                abs(self.data),
                 level=self.threshold.value,
                 step_size=1,
             )
@@ -500,21 +467,21 @@ class ThreeDViewer(widgets.Box):
 
                 # linear scale
                 if 'log' in oldv and 'log' not in newv:
-                    d = self.d0
-                    self.set_data(d, threshold=10 ** self.threshold.value)
+                    data = self.d0
+                    self.set_data(data, threshold=10 ** self.threshold.value)
 
                 # log scale
                 elif 'log' in newv and 'log' not in oldv:
-                    self.d0 = self.d
-                    d = np.log10(np.maximum(0.1, abs(self.d0)))
-                    self.set_data(d, threshold=np.log10(self.threshold.value))
+                    self.d0 = self.data
+                    data = np.log10(np.maximum(0.1, abs(self.d0)))
+                    self.set_data(data, threshold=np.log10(self.threshold.value))
                     return
             self.on_update_plot()
 
-    def set_data(self, d, threshold=None):
+    def set_data(self, data, threshold=None):
         """Check if data is complex or not
 
-        :param d: data 3d array, complex ot not, to be plotted
+        :param data: data 3d array, complex ot not, to be plotted
         :param threshold: threshold for contour, if None set to max/2
         """
 
@@ -522,12 +489,12 @@ class ThreeDViewer(widgets.Box):
         self.progress.value = 5
 
         # Save data
-        self.d = d
+        self.data = data
 
         # Change scale options depending on data
         self.toggle_phase.unobserve(self.on_change_scale)
 
-        if np.iscomplexobj(d):
+        if np.iscomplexobj(data):
             if self.toggle_phase.value == 'log10(Abs)':
                 self.toggle_phase.value = 'Abs'
             self.toggle_phase.options = ('Abs', 'Phase')
@@ -540,34 +507,34 @@ class ThreeDViewer(widgets.Box):
         # Set threshold
         self.threshold.unobserve(self.on_update_plot)
         self.colormap_range.unobserve(self.on_update_plot)
-        self.threshold.max = abs(self.d).max()
+        self.threshold.max = abs(self.data).max()
         if threshold is None:
             self.threshold.value = self.threshold.max / 2
         else:
             self.threshold.value = threshold
 
         # Set colormap
-        self.colormap_range.max = abs(self.d).max()
-        self.colormap_range.value = [0, abs(self.d).max()]
+        self.colormap_range.max = abs(self.data).max()
+        self.colormap_range.value = [0, abs(self.data).max()]
         self.threshold.observe(self.on_update_plot)
         self.colormap_range.observe(self.on_update_plot)
 
-        nz, ny, nx = self.d.shape
+        nz, ny, nx = self.data.shape
         z, y, x = np.arange(nz), np.arange(ny), np.arange(nx)
 
         # Interpolate probe to object grid
         self.rgi = RegularGridInterpolator(
             (z, y, x),
-            self.d,
+            self.data,
             method='linear',
             bounds_error=False,
             fill_value=0,
         )
 
         # Also prepare the phase gradient
-        gz, gy, gx = np.gradient(self.d)
-        a = np.maximum(abs(self.d), 1e-6)
-        ph = self.d / a
+        gz, gy, gx = np.gradient(self.data)
+        a = np.maximum(abs(self.data), 1e-6)
+        ph = self.data / a
         gaz, gay, gax = np.gradient(a)
         self.rgi_gx = RegularGridInterpolator(
             (z, y, x), ((gx - gax * ph) / (ph * a)).real,
@@ -580,10 +547,10 @@ class ThreeDViewer(widgets.Box):
             method='linear', bounds_error=False, fill_value=0)
 
         # Fix extent otherwise weird things happen
-        ipv.pylab.xlim(0, self.d.shape[0])
-        ipv.pylab.ylim(0, self.d.shape[1])
-        ipv.pylab.zlim(0, self.d.shape[2])
-        ipv.squarelim()
+        ipv.pylab.xlim(0, self.data.shape[0])
+        ipv.pylab.ylim(0, self.data.shape[1])
+        ipv.pylab.zlim(0, self.data.shape[2])
+        # ipv.squarelim()
         self.on_update_plot()
 
     def on_animate(self, v):
@@ -623,8 +590,9 @@ def plot_data(
 ):
     """
     Create figure based on the data dimensions.
-    Data in 2D is here always plotted as an image
-    Data in 3D is plotted as 3 slices or directly in 3D
+    Data in 1D is plotted as a curve
+    Data in 2D is plotted as a 2D image
+    Data in 3D is plotted with Bokeh along one axis
 
     :param data_array: np.ndarray to plot
     :param figsize: default (10, 10)
@@ -684,8 +652,9 @@ def plot_data(
     elif data_dimensions == 2:
         # Depends on log scale
         if isinstance(log, bool):
-            img = plot_2d_image(data_array, log=log, cmap=cmap,
-                                title=title, fontsize=fontsize)
+            img = plot_2d_image(
+                data_array, log=log, cmap=cmap, title=title, fontsize=fontsize
+            )
 
             # Create axis for colorbar
             cbar_ax = make_axes_locatable(ax).append_axes(
@@ -719,8 +688,9 @@ def plot_data(
                 log = scale == "logarithmic"
 
                 # Plot
-                img = plot_2d_image(data_array, log=log,
-                                    fig=fig, ax=ax, cmap=cmap)
+                img = plot_2d_image(
+                    data_array, log=log, fig=fig, ax=ax, cmap=cmap
+                )
 
                 # Create axis for colorbar
                 cbar_ax = make_axes_locatable(ax).append_axes(
@@ -744,16 +714,14 @@ def plot_data(
             scale="linear",
         ):
             """
-            Function used when using widgets with callbacks
-            on the 3D image
+            Function used when using widgets with callbacks on the 3D image
 
             :param data: 3D data array
-            :param axis: axis on which the data is projected
-             in ("x", "y", "z")
+            :param axis: axis on which the data is projected in ("x", "y", "z")
             :param index: index along param axis
-            :param data_type: plot module, phase,
-             real or imaginary part of the data
-            :param scale: log or lin
+            :param data_type: plot module, phase, real or imaginary part of
+                the data
+            :param scale: "logarithmic" for log scale, otherwise linear scale
             """
             # Project on specific index
             if axis == "x":
@@ -1007,7 +975,8 @@ def plot_2d_image(
     cmap="turbo",
     title=None,
     x_label="x",
-    y_label="y"
+    y_label="y",
+    contour=False,
 ):
     """
     Plot 2d image from 2d array.
@@ -1018,9 +987,13 @@ def plot_2d_image(
      will create a figure, used to add 2d images to 3d figures
     :param ax: axes of figure, default is None and will create axes
     :param log: True to have a logarithmic scale, False to have a linear scale
-    :param cmap: matplotlib cmap used for plot, default 'YlGnBu_r'
+    :param cmap: matplotlib cmap used for plot, default 'turbo'
     :param title: str, title for this axe
-    :return: image on ax (ax.imshow())
+    :param x_label: label on x axis
+    :param y_label: label on y axis
+    :param contour: True to plot contour of data
+
+    :return: image on ax : ax.imshow()
     """
     if not fig and not ax:
         fig, ax = plt.subplots(1, 1, figsize=(5, 5))
@@ -1028,28 +1001,25 @@ def plot_2d_image(
     scale = "logarithmic" if log else "linear"
 
     try:
-        img = ax.imshow(
-            two_d_array,
-            norm={"linear": None, "logarithmic": LogNorm()}[
-                scale],
-            cmap=cmap,
-        )
-        ax.set_xlabel(x_label, fontsize=fontsize)
-        ax.set_ylabel(y_label, fontsize=fontsize)
-        if isinstance(title, str):
-            ax.set_title(title, fontsize=fontsize + 2)
-
-        return img
-
-    except TypeError:
-        print("Using complex data, automatically switching to array module")
-
-        img = ax.imshow(
-            np.abs(two_d_array),
-            norm={"linear": None, "logarithmic": LogNorm()}[
-                scale],
-            cmap=cmap,
-        )
+        if np.iscomplex(two_d_array).any():
+            print(
+                "Using complex data, switching to array module for plot."
+            )
+            two_d_array = np.abs(two_d_array)
+        if contour:
+            img = ax.contourf(
+                two_d_array,
+                norm={"linear": None, "logarithmic": LogNorm()}[
+                    scale],
+                cmap=cmap,
+            )
+        else:
+            img = ax.imshow(
+                two_d_array,
+                norm={"linear": None, "logarithmic": LogNorm()}[
+                    scale],
+                cmap=cmap,
+            )
         ax.set_xlabel(x_label, fontsize=fontsize)
         ax.set_ylabel(y_label, fontsize=fontsize)
         if isinstance(title, str):
@@ -1073,6 +1043,8 @@ def plot_3d_slices(
     log=False,
     cmap="turbo",
     title=None,
+    contour=False,
+    sum_over_axis=False,
 ):
     """
     Plot 3 slices for 3d data.
@@ -1085,6 +1057,8 @@ def plot_3d_slices(
     :param cmap: matplotlib cmap used for plot, default 'YlGnBu_r'
     :param title: string to set as title for main plot or list of
      strings of length 3 for sub axes
+    :param contour: True to plot contour of data
+    :param sum_over_axis: True to plot sum intead of slices
     """
     if isinstance(log, bool):
         # Create figure
@@ -1097,19 +1071,26 @@ def plot_3d_slices(
         if isinstance(title, str):
             fig.suptitle(title, fontsize=fontsize + 2, y=0.95)
             titles = [None, None, None]
-        elif isinstance(title, tuple) and len(title) == 3 or isinstance(title, list) and len(title) == 3:
+        elif isinstance(title, tuple) and len(title) == 3\
+                or isinstance(title, list) and len(title) == 3:
             titles = title
         else:
             titles = [None, None, None]
 
-        # Each axis has a dimension
+        # 3D array shape
         shape = data_array.shape
 
         # Plot first image
-        two_d_array = data_array[shape[0]//2, :, :]
-        img_x = plot_2d_image(two_d_array, fig=fig, title=titles[0],
-                              ax=axs[0], log=log, cmap=cmap, fontsize=fontsize,
-                              x_label="z", y_label="y")
+        if sum_over_axis:  # Compute sum
+            two_d_array = np.sum(np.nan_to_num(data_array), axis=(0))
+
+        else:  # Get middle slice
+            two_d_array = data_array[shape[0]//2, :, :]
+        img_x = plot_2d_image(
+            two_d_array, fig=fig, title=titles[0], ax=axs[0], log=log,
+            cmap=cmap, fontsize=fontsize, x_label="z", y_label="y",
+            contour=contour
+        )
 
         # Create axis for colorbar
         cbar_ax = make_axes_locatable(axs[0]).append_axes(
@@ -1119,10 +1100,16 @@ def plot_3d_slices(
         fig.colorbar(mappable=img_x, cax=cbar_ax)
 
         # Plot second image
-        two_d_array = data_array[:, shape[1]//2, :]
-        img_y = plot_2d_image(two_d_array, fig=fig, title=titles[1],
-                              ax=axs[1], log=log, cmap=cmap, fontsize=fontsize,
-                              x_label="z", y_label="x")
+        if sum_over_axis:  # Compute sum
+            two_d_array = np.sum(np.nan_to_num(data_array), axis=(1))
+
+        else:  # Get middle slice
+            two_d_array = data_array[:, shape[1]//2, :]
+        img_y = plot_2d_image(
+            two_d_array, fig=fig, title=titles[1], ax=axs[1], log=log,
+            cmap=cmap, fontsize=fontsize, x_label="z", y_label="x",
+            contour=contour
+        )
 
         # Create axis for colorbar
         cbar_ax = make_axes_locatable(axs[1]).append_axes(
@@ -1132,10 +1119,16 @@ def plot_3d_slices(
         fig.colorbar(mappable=img_y, cax=cbar_ax)
 
         # Plot third image
-        two_d_array = data_array[:, :, shape[2]//2]
-        img_z = plot_2d_image(two_d_array, fig=fig, title=titles[2],
-                              ax=axs[2], log=log, cmap=cmap, fontsize=fontsize,
-                              x_label="y", y_label="x")
+        if sum_over_axis:  # Compute sum
+            two_d_array = np.sum(np.nan_to_num(data_array), axis=(2))
+
+        else:  # Get middle slice
+            two_d_array = data_array[:, :, shape[2]//2]
+        img_z = plot_2d_image(
+            two_d_array, fig=fig, title=titles[2], ax=axs[2], log=log,
+            cmap=cmap, fontsize=fontsize, x_label="y", y_label="x",
+            contour=contour
+        )
 
         # Create axis for colorbar
         cbar_ax = make_axes_locatable(axs[2]).append_axes(
@@ -1151,78 +1144,28 @@ def plot_3d_slices(
     else:
         @interact(
             scale=widgets.ToggleButtons(
-                options=["linear", "logarithmic"],
-                value="linear",
+                options=[
+                    ("linear", False),
+                    ("logarithmic", True),
+                ],
+                value=False,
                 description='Scale',
                 disabled=False,
                 style={'description_width': 'initial'}),
             figsize=fixed(figsize)
         )
         def plot_with_interactive_scale(scale, figsize):
-            # Create figure
-            if not figsize:
-                figsize = (15, 7)
-
-            fig, axs = plt.subplots(1, 3, figsize=figsize)
-
-            if isinstance(title, str):
-                fig.suptitle(title, fontsize=fontsize + 2)
-                titles = [None, None, None]
-            elif isinstance(title, tuple) and len(title) == 3 or isinstance(title, list) and len(title) == 3:
-                titles = title
-            else:
-                titles = [None, None, None]
-
-            # Each axis has a dimension
-            shape = data_array.shape
-
-            # Get scale
-            log = scale == "logarithmic"
-
             try:
-                # Create image slice along x
-                two_d_array = data_array[shape[0]//2, :, :]
-                img_x = plot_2d_image(two_d_array, fig=fig, title=titles[0],
-                                      ax=axs[0], log=log, cmap=cmap,
-                                      fontsize=fontsize, x_label="z", y_label="y")
-
-                # Create axis for colorbar
-                cbar_ax = make_axes_locatable(axs[0]).append_axes(
-                    position='right', size='5%', pad=0.1)
-
-                # Create colorbar
-                fig.colorbar(mappable=img_x, cax=cbar_ax)
-
-                # Create image slice along y
-                two_d_array = data_array[:, shape[1]//2, :]
-                img_y = plot_2d_image(two_d_array, fig=fig, title=titles[1],
-                                      ax=axs[1], log=log, cmap=cmap,
-                                      fontsize=fontsize, x_label="z", y_label="x")
-
-                # Create axis for colorbar
-                cbar_ax = make_axes_locatable(axs[1]).append_axes(
-                    position='right', size='5%', pad=0.1)
-
-                # Create colorbar
-                fig.colorbar(mappable=img_y, cax=cbar_ax)
-
-                # Create image slice along z
-                two_d_array = data_array[:, :, shape[2]//2]
-                img_z = plot_2d_image(two_d_array, fig=fig, title=titles[2],
-                                      ax=axs[2], log=log, cmap=cmap,
-                                      fontsize=fontsize, x_label="y", y_label="x")
-
-                # Create axis for colorbar
-                cbar_ax = make_axes_locatable(axs[2]).append_axes(
-                    position='right', size='5%', pad=0.1)
-
-                # Create colorbar
-                fig.colorbar(mappable=img_z, cax=cbar_ax)
-
-                # Show figure
-                plt.tight_layout()
-                plt.show()
-                plt.close()
+                plot_3d_slices(
+                    data_array=data_array,
+                    fontsize=fontsize,
+                    figsize=figsize,
+                    log=scale,
+                    cmap=cmap,
+                    title=title,
+                    contour=contour,
+                    sum_over_axis=sum_over_axis,
+                )
             except IndexError:
                 plt.close()
                 print("Is this a 3D array?")
